<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/core
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2022 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */
namespace JchOptimize\Core\Css\Callbacks;

use JchOptimize\Core\Css\Parser;
use JchOptimize\Core\Helper;
use function strpos;
class BackgroundImagesLazyLoad extends \JchOptimize\Core\Css\Callbacks\AbstractCallback
{
    public function processMatches($matches, $context) : string
    {
        if (\preg_match('#' . Parser::CSS_URL_CP(\true) . '#', $matches[0], $m)) {
            //Don't need to lazy-load data-image
            if (strpos($m[1], 'data:image') !== 0) {
                //Remove the background image
                $matches[0] = \str_replace($m[0], '', $matches[0]);
                //Remove any empty background declarations
                $matches[0] = \preg_replace('#background(?:-image)?\\s*+:\\s*+;#', '', $matches[0]);
                //Add the lazy-loaded image to CSS
                $css = $matches[0] . '.' . Helper::cssSelectorsToClass($matches[3]) . '.jch-lazyloaded{background-image:' . $m[0] . '}';
                return $css;
            }
        }
        return $matches[0];
    }
}
